@@ -49,7 +49,7 @@ public class AttachmentSerializer {
     // http://tools.ietf.org/html/rfc2387
     private static final String DEFAULT_MULTIPART_TYPE = "multipart/related";
 
-    private String contentTransferEncoding = "binary";
+    private String contentTransferEncoding = AttachmentUtil.BINARY;
 
     private Message message;
     private String bodyBoundary;
@@ -296,7 +296,7 @@ public class AttachmentSerializer {
                 writer.write("\r\n--");
                 writer.write(bodyBoundary);
 
-                Map<String, List<String>> headers = null;
+                final Map<String, List<String>> headers;
                 Iterator<String> it = a.getHeaderNames();
                 if (it.hasNext()) {
                     headers = new LinkedHashMap<>();
